@@ -4,8 +4,8 @@ General tools useful to create NeXus format files.
 from __future__ import annotations
 
 __author__ = "Diamond Light Source - Scientific Software"
-__email__ = "scientificsoftware@diamond.ac.uk"
-__version__ = "0.6.16"
+__email__ = "data_analysis@diamond.ac.uk"
+__version__ = "0.6.18"
 __version_tuple__ = tuple(int(x) for x in __version__.split("."))
 
 import logging
